#pragma once

#include "common.h"
#include "Context.h"
class HField;
class BMPacket;

namespace WorldMachine {
/**
A generic class for resampling a bitmap or heightfield at a new dimension.
*/
class Rescaler
{
public:
	Rescaler(void);
	~Rescaler(void);

	static Coord LimitSize(const Coord &size, int limit);
	static Coord LimitSize(const Coord &size, const Coord &limit); // optimally make sure we fit to a rectangle-sized limit

	static HFPointer RescaleHF(HFPointer input, Coord  dims);		///< Rescale input heightfield to the given dimensions and return new one
	static HFPointer RescaleHF(HFPointer input, PacketContext &dims);	///< Rescale input heightfield to the given dimensions and return new one

	static BMPointer RescaleBMP(BMPointer input, Coord  dims);		///< Rescale input bitmap to the given dimensions and return new one
	static BMPointer RescaleBMP(BMPointer input, PacketContext &dims);	///< Rescale input bitmap to the given dimensions and return new one

	static void Nearest2(HField &src, HField &dest);
	static void Bilinear2(HField &src, HField &dest);
	static void Bicubic2(HField &src, HField &dest);
	static void BilinearBMP(BMPacket &src, BMPacket &dest);

	static void FractalInterp(HField &src, HField &dest, const float smoothness = 0.5f);
protected:
    
};

}