#ifndef _BITMAP_H
#define _BITMAP_H

#include "common.h"
//#include "gfx_def.h"

class istream;
class ostream;

// Change this define whenever disk format changes
#define BITMAP_VERSION		4

enum IMG_DAT_TYPE { RGB16_555, RGB16_565, RGB24, RGBA32, ALPHA8 };

const int FLEN = 64;

inline bool IsRGB(IMG_DAT_TYPE a) { return (a <= RGBA32); };

/** Class for dealing with simple bitmaps.

\deprecated

This class is old, deprecated, and not used internally in World Machine except in a few cases (usually dealing with toolbar icons, etc).
As such, its use is not recommended. A BMPacket is slightly more heavyweight but will generally serve you better.

  */
class Bitmap {
public:
// Housekeeping
	Bitmap();
	~Bitmap();
	Bitmap( Bitmap &other);
	Bitmap &operator=( const Bitmap &other );

// Native load/save functions, for individual or stream loading
	void CreateBlank(Coord size, IMG_DAT_TYPE format); // Initialize bitmap
	bool Load(std::istream &in);	// Load a .gfx file from an input stream
	bool Save(std::ostream &out);	// Save this bitmap to an output stream

// Import/Export functions
	bool LoadTGA(const char *fname, IMG_DAT_TYPE convert = RGB16_565);	// load a .TGA from disk
	bool LoadBMP(const char *fname, IMG_DAT_TYPE convert = RGB16_565);	// load and optionally convert a BMP
	bool SaveTGA(const char *fname); // Save the image only to a .tga file

// Manipulation functions for changing the image data contained here
	void ConvertRGB(IMG_DAT_TYPE new_type);	// General purpose intra-RGB format converter
	IMG_DAT_TYPE GetFormat() { return data_type; };
	bool MakeFromBuffer(Coord size, IMG_DAT_TYPE format, void *input); // **NOTE: input buffer turned over to bitmap to manage!

	/// Returns a bitmap-packet from this source. The result's life is independent of the original bitmap.
	/// Note that this function will only return a result for a 32bit RGBA bitmap.
	BMPointer ExportBMPacket(); 

// Public data and access functions
	char *Getname() { return name; };	// Return image name
	byte *Getdata() { return data; };			// Return image data pointer
	byte *GetdataRow(int row) { return data + size_t(psize())*gwidth*row; };
	size_t GetSize() { return size; };
	int width, height;		// Real size of image data per cell
	int gwidth, gheight;	// Total Width and Height of all cells in this bitmap

	dword pixel32(int x, int y);	// Grab 32bit color value at location
	int	psize() { // return pixel size
		switch (data_type) {
		case RGBA32:
			return 4;
		case RGB24:
			return 3;
		case ALPHA8:
			return 1;
		default:
			return 2;
		}
	};

protected:
/////////////////////
	virtual void clear();		// Deletes all data for this object
	virtual void clone(const Bitmap &other); // Makes a deep copy of the other bmp.
	size_t size;			// Size of data array IN BYTES
	byte *data;			// Pointer to data array
	IMG_DAT_TYPE data_type;	// Type of image data.
	char name[FLEN];	// Name of this bitmap in library
};

#endif