/** 

@mainpage World Machine Plugin Developers Guide

@section sec_intro Introduction

Welcome to the PDK for World Machine!

This documentation provides everything you need to do to write your own plugin devices to operate within World Machine.

@section install_sec Installation

The World Machine source library has been setup to make it as painless as possible to integrate World Machine into your code.
Simply unzip (or self-extract) the development kit to its own folder. It is organized along these divisions:

	- @c /docs			These document files
	- @c /examples		Example projects that use the World Machine core
	- @c /lib			All required library files
	- @c /src			Headers
	.

World Machine was developed in unmanaged C++ using Microsoft Visual Studio 2010. If you are using a different version of the MSVC compiler,
full source is provided (in the full license model) to recompile to suit your needs.

@section sec_arch High Level Architecture

@par World Machine Engine

World Machine's internal engine architecture can be roughly divided into three sections:

	- @ref devices		to learn about World Machine's graph-based processing model
	- @ref data			to learn about how data is represented within World Machine
	- Device Building tools for processing the world. You don't have to worry about this at all while writing a plugin!
	.

Together these three modules load, represent, process, and store all user data.

@section sec_op Operation

The operation of World Machine can be summed up simply:

	- A set of Device class objects encapsulate actions taken to generate, modify, or export terrain (or other) data.
	- The DeviceWorld is a collection of Devices that are linked together into a network that data flows through when activated
	- A DeviceWorldBuilder traverses a DeviceWorld, building the Devices within it.
		- Each device is build by having its Activate() function called.
		- A BuildContext is presented to Activate() to give it details about the world in which its building and other relevant details.
		.
	.

@section using_sec Details about...

	- @ref threading
	- @ref multiscale
	- @ref page_using_plugin	Writing a new plugin for World Machine
	- @ref page_using_api		Using World Machine in your game, simulation, or application
	.

*/


/** @page multiscale Multiscale Processing

World Machine provides assistance classes for multiscale programming.

*/


/** @defgroup memory Memory Management

There are a few things to note about memory management in WM:

 -# The host application must provide a malloc and free -style pair of allocation routines to be used
    by all components within WM. Anything passed around internally must be alloc'd with WMAlloc.
 -# If the WMAlloc() function cannot allocate memory, it throws a WMException error with OUT_OF_MEMORY state.
    Catching this allows you to appropriately handle out of memory conditions in your code. The WM Core assumes memory allocations cannot fail.
 -# Data packets are reference-counted and an intrusive smart pointer system is used to manage resources. Although resources are smart-pointer wrapped internally,
    you may freely pass raw pointers (or references) within your call stack -- if you are holding onto pointers internally, it would be best to use the smart pointers provided.
 -# Packets of data managed by the WMPacket system have an application-level paging mechanism to allow WM to better control
    what memory is allowed to stay in memory at once. This is required since high resolution heightfields are very large.
	This is mostly transparent to you, the plugin writer, as the WM core makes sure you are never handed a paged out heightfield.

*/


/** @defgroup threading Multithreading

Multithreading support within World Machine

*/




/** @defgroup data Data Packets

Data packets derive from WMPacket and flow information through the network.
By deriving from WMPacket, any type of information can be carried.

*/